@@ -2,4 +2,5 @@ library(bayesPop)
 source('test_functions.R')
 
 test.prediction()
-#test.expressions()
+test.expressions()
+test.expressions.with.VE()
